@@ -632,9 +632,10 @@ async fn collection(
     path: web::Path<CollectionPath>,
     token: Option<ValidToken>,
     state: web::Data<State>,
+    req: HttpRequest,
 ) -> Result<HttpResponse, StateError> {
     match token {
-        Some(token) => edit_collection(path, token, state.clone())
+        Some(token) => edit_collection(path, token, state.clone(), req)
             .await
             .stateful(&state),
         None => view_collection(path, state.clone()).await.stateful(&state),
@@ -668,7 +669,10 @@ async fn edit_collection(
     path: web::Path<CollectionPath>,
     token: ValidToken,
     state: web::Data<State>,
+    req: HttpRequest,
 ) -> Result<HttpResponse, Error> {
+    let qr = qr(&req, &path, &state);
+
     let collection = match state.store.collection(&path).await? {
         Some(collection) => collection,
         None => return Ok(to_404(&state)),
@@ -687,6 +691,7 @@ async fn edit_collection(
                 &entries,
                 &token,
                 &state,
+                &qr,
             )
         },
         HttpResponse::Ok(),
@@ -824,6 +829,48 @@ async fn delete_entry(
     Ok(to_edit_page(entry_path.collection, &token, &state))
 }
 
+fn qr(req: &HttpRequest, path: &web::Path<CollectionPath>, state: &web::Data<State>) -> String {
+    let host = req.head().headers().get("host").unwrap();
+
+    let url = format!(
+        "https://{}{}",
+        host.to_str().unwrap(),
+        state.public_collection_path(path.collection)
+    );
+
+    let code = qrcodegen::QrCode::encode_text(&url, qrcodegen::QrCodeEcc::Low).unwrap();
+
+    to_svg_string(&code, 4)
+}
+
+fn to_svg_string(qr: &qrcodegen::QrCode, border: i32) -> String {
+    assert!(border >= 0, "Border must be non-negative");
+    let mut result = String::new();
+    // result += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
+    // result += "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n";
+    let dimension = qr
+        .size()
+        .checked_add(border.checked_mul(2).unwrap())
+        .unwrap();
+    result += &format!(
+		"<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" viewBox=\"0 0 {0} {0}\" stroke=\"none\">\n", dimension);
+    result += "\t<rect width=\"100%\" height=\"100%\" fill=\"#FFFFFF\"/>\n";
+    result += "\t<path d=\"";
+    for y in 0..qr.size() {
+        for x in 0..qr.size() {
+            if qr.get_module(x, y) {
+                if x != 0 || y != 0 {
+                    result += " ";
+                }
+                result += &format!("M{},{}h1v1h-1z", x + border, y + border);
+            }
+        }
+    }
+    result += "\" fill=\"#000000\"/>\n";
+    result += "</svg>\n";
+    result
+}
+
 #[tracing::instrument(name = "Delete Collection")]
 async fn delete_collection(
     path: web::Path<CollectionPath>,
